///////////////////////////////////////////////////////////////////
// FILE:      main.cpp
// CONTAINS:  template code for CPSC 314, Jan 2005, assignment 1
///////////////////////////////////////////////////////////////////

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

float Angle = 0;

///////////////////////////////////////////////////////////////////
// FUNC:  displayCallback()
// DOES:  clears screen, draws the world, swaps buffers
///////////////////////////////////////////////////////////////////																										

void displayCallback(void)
{
	float eye[] = {0,4,20};
	float center[] = {0,0,0};
	float up[] = {0,1,0};

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	  // position the camera in the world
	gluLookAt(eye[0],eye[1],eye[2],
		center[0],center[1],center[2],
		up[0],up[1], up[2]);

	  // animate the global scene rotation
	glRotatef(Angle,1,1,0);

	  // draw a wire-frame sphere;  turn off lighting in order to get solid shading
	glDisable(GL_LIGHTING);
	glColor3f(1,1,1);
	glutWireSphere(1,20,10);     // sphere: r=1, 20 slices, 10 stacks
	glEnable(GL_LIGHTING);

	glPushMatrix();
	  // draw some colored teapots
	glTranslatef(-3,0,1);
	glColor3f(1,0.5,0.5);      
	glutSolidTeapot(1.0);

	glTranslatef(3,0,0);
	glColor3f(0.5,1,0.5);      
	glutSolidTeapot(1.0);
	
	glTranslatef(3,0,0);
	glColor3f(0.5,0.5,1);      
	glutSolidTeapot(1.0);

	glPopMatrix();

	  // draw ground plane
	glColor3f(1,1,1);
	glNormal3f(0,1,0);
	glBegin(GL_POLYGON);
	glVertex3f(-10,0,-10);
	glVertex3f(-10,0,10);
	glVertex3f(10,0,10);
	glVertex3f(10,0,-10);
	glEnd();

	glutSwapBuffers();
}

///////////////////////////////////////////////////////////////////
// FUNC:  keyCallback()
// DOES:  clears screen, draws the world, swaps buffers
///////////////////////////////////////////////////////////////////																										

void keyCallback(unsigned char key, int x, int y)
{
	switch(key) {
	case 'q': 
		exit(0);
		break;
	default:
		break;
	}
	glutPostRedisplay();
}

///////////////////////////////////////////////////////////////////
// FUNC:  myReshape()
// DOES:  this is called if the on-screen window is resized
///////////////////////////////////////////////////////////////////																										

void myReshape(int w, int h)
{
   glViewport(0, 0, w, h);
   glMatrixMode(GL_PROJECTION); 
   glLoadIdentity();
   gluPerspective(35.0, (GLfloat) w/h, 1.0, 100.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

///////////////////////////////////////////////////////////////////
// FUNC:  idleCallback()
// DOES:  glut calls this when idle;  often used for animation
///////////////////////////////////////////////////////////////////																										

void idleCallback()
{
	static float delta = 0.5;
	Angle += delta;
	if (Angle>30 || Angle<-30)
		delta *= -1;
	glutPostRedisplay();
}

///////////////////////////////////////////////////////////////////
// FUNC:  init()
// DOES:  initializes default light source and surface material
///////////////////////////////////////////////////////////////////																										

void init(void) 
{
   glClearColor (0.2f, 0.3f, 0.4f, 1.0);
   glShadeModel (GL_SMOOTH);

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_COLOR_MATERIAL);
   glEnable(GL_NORMALIZE);
}

///////////////////////////////////////////////////////////////////
// FUNC:  main()
// DOES:  intializes glut window, calls init(), then hands over control to glut
///////////////////////////////////////////////////////////////////																										

int main(int argc, char **argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(800, 800);
   glutCreateWindow("Light Lab");
   glutReshapeFunc(myReshape);
   glutDisplayFunc(displayCallback);
   glutKeyboardFunc(keyCallback);
   glutIdleFunc(idleCallback);
   init();
   glutMainLoop();
   return 0;
}
